attribute highp vec4 a_position;
attribute lowp vec4 a_color;
attribute mediump vec2 a_texCoord;
attribute mediump vec3 a_normal;

varying mediump vec3 v_normal;
varying mediump float v_vdn;
varying mediump vec3 v_vpos;
varying mediump vec3 v_vvec;

uniform mediump mat4 u_viewMatrix;
uniform highp mat4 u_modelViewProjectionMatrix;
uniform highp mat4 u_modelViewMatrix;
uniform lowp vec4 RimDirection;

varying lowp vec4 v_color;
varying mediump vec2 v_texCoord;

mat3 rotationMatrix(vec3 axis, float angle)
{
    axis = normalize(axis);
    float s = sin(angle);
    float c = cos(angle);
    float oc = 1.0 - c;
    
    return mat3(oc * axis.x * axis.x + c,             oc * axis.x * axis.y - axis.z * s,   oc * axis.z * axis.x + axis.y * s,
                       oc * axis.x * axis.y + axis.z * s,  oc * axis.y * axis.y + c,             oc * axis.y * axis.z - axis.x * s,
                       oc * axis.z * axis.x - axis.y * s,   oc * axis.y * axis.z + axis.x * s,  oc * axis.z * axis.z + c);
}

void main() {

	//vec3 RimDirectionModelSpace = (u_modelViewMatrix * RimDirection).xyz;

	v_texCoord = a_texCoord;

	gl_Position = u_modelViewProjectionMatrix * a_position;

	// Convert normal to view space
	v_normal = (u_modelViewMatrix * (vec4(a_normal, 0.0))).xyz;

	// Offset normals according to RimDirection uniform
	mat3 v_normalRotationOffset = rotationMatrix(normalize(RimDirection.xyz), RimDirection.w * 3.141592654/180.0);
	v_normal = v_normalRotationOffset * v_normal;
	
	v_vpos = vec3(u_modelViewMatrix * a_position);
	v_vvec = normalize(-v_vpos);

	// Difference between normal and view vector
	v_vdn = 1.0 - max(dot(v_vvec, v_normal), 0.0);

}
